@@ -35,14 +35,6 @@ has current_node =>
 	required => 0,
 );
 
-has error_str =>
-(
-	default  => sub {return ''},
-	is       => 'rw',
-	isa      => Str,
-	required => 0,
-);
-
 has grammar =>
 (
 	default  => sub {return ''},
@@ -51,22 +43,6 @@ has grammar =>
 	required => 0,
 );
 
-has marpa_error_count =>
-(
-	default  => sub{return 0},
-	is       => 'rw',
-	isa      => Int,
-	required => 0,
-);
-
-has perl_error_count =>
-(
-	default  => sub{return 0},
-	is       => 'rw',
-	isa      => Int,
-	required => 0,
-);
-
 has re =>
 (
 	default  => sub {return ''},
@@ -123,7 +99,7 @@ has warning_str =>
 	required => 0,
 );
 
-our $VERSION = '0.80';
+our $VERSION = '0.81';
 
 # ------------------------------------------------
 
@@ -222,6 +198,33 @@ sub as_string
 
 # ------------------------------------------------
 
+sub find
+{
+	my($self, $target) = @_;
+
+	die "Method find() takes a defined value as the parameter\n" if (! defined $target);
+
+	my(@found);
+	my($meta);
+
+	for my $node ($self -> tree -> traverse)
+	{
+		next if ($node -> is_root);
+
+		$meta = $node -> meta;
+
+		if (index($$meta{text}, $target) >= 0)
+		{
+			push @found, $$meta{uid};
+		}
+	}
+
+	return [@found];
+
+} # End of find.
+
+# ------------------------------------------------
+
 sub get
 {
 	my($self, $wanted_uid)	= @_;
@@ -229,7 +232,7 @@ sub get
 
 	if (! defined($wanted_uid) || ($wanted_uid < 1) || ($wanted_uid > $self -> uid) )
 	{
-		die "Method get() takes a uid parameter in the range 1 .. $max_uid \n";
+		die "Method get() takes a uid parameter in the range 1 .. $max_uid\n";
 	}
 
 	my($meta);
@@ -275,7 +278,6 @@ sub parse
 
 	# Emulate parts of new(), which makes things a bit earier for the caller.
 
-	$self -> error_str('');
 	$self -> re($opts{re})				if (defined $opts{re});
 	$self -> verbose($opts{verbose})	if (defined $opts{verbose});
 	$self -> warning_str('');
@@ -306,22 +308,20 @@ sub parse
 		{
 			$result = 1;
 
-			if (! $self -> error_str)
-			{
-				$self -> error_str('Marpa error: Parse failed');
+			my($message) = "Error: Marpa parse failed. ";
 
-				say $self -> error_str if ($self -> verbose);
-			}
+			say $message if ($self -> verbose);
+
+			die $message;
 		}
 	}
 	catch
 	{
-		$result = 1;
+		my($message) = "Error: Marpa parse failed. $_";
 
-		$self -> marpa_error_count($self -> marpa_error_count + 1);
-		$self -> error_str("Marpa error: Parse failed. $_");
+		say $message if ($self -> verbose);
 
-		say $self -> error_str if ($self -> verbose);
+		die $message;
 	};
 
 	# Return 0 for success and 1 for failure.
@@ -411,7 +411,7 @@ sub _process
 		$lexeme	= $self -> recce -> literal($start, $span);
 		$pos	= $self -> recce -> lexeme_read($event_name);
 
-		die "Marpa lexeme_read($event_name) rejected lexeme |$lexeme|\n" if (! defined $pos);
+		die "Marpa lexeme_read($event_name) rejected lexeme '$lexeme'\n" if (! defined $pos);
 
 		say "event_name: $event_name. lexeme: $lexeme. " if ($self -> verbose > 1);
 
@@ -420,19 +420,22 @@ sub _process
 
 	my($message);
 
-	if ($self -> recce -> exhausted)
+	if (my $status = $self -> recce -> ambiguous)
+	{
+		my($terminals)	= $self -> recce -> terminals_expected;
+		$terminals		= ['(None)'] if ($#$terminals < 0);
+		$message		= "Marpa warning. Parse ambiguous. Status: $status. Terminals expected: " . join(', ', @$terminals);
+	}
+	elsif ($self -> recce -> exhausted)
 	{
 		# See https://metacpan.org/pod/distribution/Marpa-R2/pod/Exhaustion.pod#Exhaustion
 		# for why this code is exhaustion-loving.
 
 		$message = 'Marpa parse exhausted';
 	}
-	elsif (my $status = $self -> recce -> ambiguous)
-	{
-		my($terminals)	= $self -> recce -> terminals_expected;
-		$terminals		= ['(None)'] if ($#$terminals < 0);
-		$message		= "Marpa warning. Parse ambiguous. Status: $status. Terminals expected: " . join(', ', @$terminals);
 
+	if ($message)
+	{
 		$self -> warning_str($message);
 
 		say $message if ($self -> verbose);
@@ -488,9 +491,6 @@ sub reset
 	$self -> tree(Tree -> new('Root') );
 	$self -> tree -> meta({text => 'Root', uid => 0});
 	$self -> current_node($self -> tree);
-	$self -> error_str('');
-	$self -> marpa_error_count(0);
-	$self -> perl_error_count(0);
 	$self -> uid(0);
 	$self -> warning_str('');
 
@@ -529,20 +529,21 @@ sub set
 
 sub _string2re
 {
-	my($self, $candidate) = @_;
+	my($self, $raw_re) = @_;
 
 	my($re);
 
 	try
 	{
-		$re = does($candidate, 'Regexp') ? $candidate : qr/$candidate/;
+		$re = does($raw_re, 'Regexp') ? $raw_re : qr/$raw_re/;
 	}
 	catch
 	{
-		$re = '';
+		my($message) = "Error: Perl cannot convert $raw_re into qr/.../ form";
 
-		$self -> perl_error_count($self -> perl_error_count + 1);
-		$self -> error_str($self -> test_count . ": Perl cannot convert $candidate into qr/.../ form");
+		say $message if ($self -> verbose);
+
+		die $message;
 	};
 
 	return $re;
@@ -622,8 +623,6 @@ This is scripts/synopsis.pl:
 
 	say "Original:  $re. Result: $result. (0 is success)";
 	say "as_string: $as_string";
-	say 'Perl error count:  ', $parser -> perl_error_count;
-	say 'Marpa error count: ', $parser -> marpa_error_count;
 
 And its output:
 
@@ -656,8 +655,6 @@ And its output:
 	close_parenthesis       7  )
 	Original:  (?^i:Perl|JavaScript). Result: 0. (0 is success)
 	as_string: (?^i:Perl|JavaScript|C++)
-	Perl error count:  0
-	Marpa error count: 0
 
 Note: The 1st tree is printed due to verbose => 1 in the call to L</new([%opts])>, while the 2nd
 is due to the call to L</print_raw_tree()>. The columnar output is due to the call to
@@ -763,59 +760,45 @@ The uid of the node to update.
 
 =back
 
+The code calls C<die()> if %opts does not have these 2 keys, or if either value is undef.
+
 See scripts/synopsis.pl for sample code.
 
 Note: Calling C<append()> never changes the uids of nodes, so repeated calling of C<append()> with
 the same C<uid> will apply more and more updates to the same node.
 
-See also L</get(%opts)>, L</prepend(%opts)>, L</set(%opts)> and t/get.set.t.
+See also L</prepend(%opts)>, L</set(%opts)> and t/get.set.t.
 
 =head2 as_string()
 
 Returns the parsed regexp as a string. The string contains all edits applied with methods such as
 L</append(%opts)>.
 
-=head2 error_str()
+=head2 find($string)
 
-Returns the last error, as a string.
+Returns an arrayref of node uids whose text contains the given string.
 
-Errors will be in 1 of 2 categories:
+The code calls C<die()> if $target is undef.
 
-=over 4
+If the arrayref is empty, there were no matches.
 
-=item o Perl errors
+This method uses the Perl C<index()> function to test if $string is a substring of the text of each
+node. Regexps are not used by this method.
 
-These arise when Perl cannot interpret the string form of the regexp supplied by you, when the code
-checks it using qr/$re/.
+See scripts/play.pl and t/get.set.t for sample usage of C<find()>.
 
-=item o Marpa errors
-
-These arise when the BNF within the module is such that the string form of the regexp cannot be
-parsed by Marpa.
-
-If you can use the regexp in Perl code, then you should never get this error. In other words, if
-Perl accepts the regexp and the module does not, then the BNF in this module is wrong (barring bugs
-in Perl of course).
-
-=back
-
-See also L</marpa_error_count()>, L</perl_error_count()> and L</warning_str()>.
+See also L</get($uid)>.
 
 =head2 get($uid)
 
-Get the text of the node whose uid is $uid.
+Get the text of the node with the given $uid.
+
+The code calls C<die()> if $uid is undef, or outside the range 1 .. $self -> uid. The latter value
+is the highest uid so far assigned to any node.
 
 Returns undef if the given $uid is not found.
 
-See also L</append(%opts)>, L</prepend(%opts)> and L</set(%opts)>.
-
-=head2 marpa_error_count()
-
-Returns an integer count of errors detected by Marpa. This value should always be 0.
-
-See also L</error_str()>, L</perl_error_count()> and L</warning_str()>.
-
-Used basically for debugging.
+See also L</find($string)>.
 
 =head2 new([%opts])
 
@@ -834,14 +817,6 @@ The hash C<%opts> takes the same (key => value) pairs as L</new()> does.
 
 See L</Constructor and Initialization> for details.
 
-=head2 perl_error_count()
-
-Returns an integer count of errors detected by perl. This value should always be 0.
-
-See also L</error_str()> , L</marpa_error_count()> and L</warning_str()>.
-
-Used basically for debugging.
-
 =head2 prepend(%opts)
 
 Prepend some text to the text of a node.
@@ -860,10 +835,12 @@ The uid of the node to update.
 
 =back
 
+The code calls C<die()> if %opts does not have these 2 keys, or if either value is undef.
+
 Note: Calling C<prepend()> never changes the uids of nodes, so repeated calling of C<prepend()> with
 the same C<uid> will apply more and more updates to the same node.
 
-See also L</append(%opts)>, L</get(%opts)> and L</set(%opts)> and t/get.set.t.
+See also L</append(%opts)>, L</set(%opts)>, and t/get.set.t.
 
 =head2 print_cooked_tree()
 
@@ -913,7 +890,9 @@ The uid of the node to update.
 
 =back
 
-See also L</append(%opts)>, L</prepend(%opts)> and L</get(%opts)>.
+The code calls C<die()> if %opts does not have these 2 keys, or if either value is undef.
+
+See also L</append(%opts)> and L</prepend(%opts)>.
 
 =head2 tree()
 
@@ -944,9 +923,13 @@ Note: C<verbose> is a parameter to L</new([%opts])>.
 
 =head2 warning_str()
 
-Returns the last Marpa warning, as a string.
+Returns the last Marpa warning.
 
-See also L</error_str()>, L</perl_error_count()> and L</marpa_error_count()>.
+In short, Marpa will always report 'Marpa parse exhausted' in warning_str() if the parse is not
+ambiguous, but do not worry - I<this is not an error>.
+
+See L<After calling parse(), warning_str() contains the string '... Parse ambiguous ...'|/FAQ> and
+L<Is this a (Marpa) exhaustion-hating or exhaustion-loving app?|/FAQ>.
 
 =head1 FAQ
 
@@ -1023,24 +1006,16 @@ See t/get.set.t for sample code.
 
 =back
 
-=head2 What is the purpose of this module?
+=head2 Does this module ever use \Q...\E to quote regexp metacharacters?
 
-=over 4
-
-=item o To provide a stand-alone parser for regexps
-
-=item o To help me learn more about regexps
-
-=item o To become, I hope, a replacement for the horrendously complex L<Regexp::Assemble>
-
-=back
+No.
 
 =head2 What is the format of the nodes in the tree build by this module?
 
 Each node's C<name> is the name of the Marpa-style event which was triggered by detection of
 some C<text> within the regexp.
 
-Each node's C<meta> method returns a hashref with these (key => value) pairs:
+Each node's C<meta()> method returns a hashref with these (key => value) pairs:
 
 =over 4
 
@@ -1052,15 +1027,19 @@ This is the text within the regexp which triggered the event just mentioned.
 
 This is the unqiue id of the 'current' node.
 
-This <uid> is often used by you to specify which node to work on.
+This C<uid> is often used by you to specify which node to work on.
 
 =back
 
 See also the source code for L</print_cooked_tree()> and L</print_raw_tree()> for ideas on how to
-use this object.
+use the tree.
 
 See the L</Synopsis> for sample code and a report after parsing a tiny regexp.
 
+=head2 Does the root node in the tree ever hold useful information?
+
+No. Always ignore it.
+
 =head2 Does this module interpret regexps in any way?
 
 No. You have to run your own Perl code to do that. This module just parses them into a data
@@ -1091,20 +1070,38 @@ urls in question.
 =head2 So which version of Perl is supported?
 
 I'm (2018-01-14) using Perl V 5.20.2 and making the BNF match the Perl regexp docs listed in
-</References> below.
+L</References> below.
+
+=head2 After calling parse(), warning_str() contains the string '... Parse ambiguous ...'
+
+This is almost certainly a error with the BNF, although of course it may be an error will an
+exceptionally-badly formed regexp.
+
+Report it via L<https://rt.cpan.org/Public/Dist/Display.html?Name=Regexp-Parsertron>, and please
+ include the regexp in the report. Thanx!
 
 =head2 Is this a (Marpa) exhaustion-hating or exhaustion-loving app?
 
 Exhaustion-loving.
 
-In short, Marpa will always report 'Marpa parse exhausted', but I<this is not an error>.
-
 See L<https://metacpan.org/pod/distribution/Marpa-R2/pod/Exhaustion.pod#Exhaustion>
 
 =head2 Will this code be modified to run under L<Marpa::R3> when the latter is stable?
 
 Yes.
 
+=head2 What is the purpose of this module?
+
+=over 4
+
+=item o To provide a stand-alone parser for regexps
+
+=item o To help me learn more about regexps
+
+=item o To become, I hope, a replacement for the horrendously complex L<Regexp::Assemble>
+
+=back
+
 =head1 Scripts
 
 This diagram indicates the flow of logic from script to script:
@@ -1164,6 +1161,10 @@ L<http://perldoc.perl.org/perlrebackslash.html>
 
 L<http://www.nntp.perl.org/group/perl.perl5.porters/2016/02/msg234642.html>
 
+L<https://code.activestate.com/lists/perl5-porters/209610/>
+
+L<https://stackoverflow.com/questions/46200305/a-strict-regular-expression-for-matching-chemical-formulae>
+
 =head1 See Also
 
 L<Graph::Regexp>
@@ -1204,12 +1205,6 @@ Version numbers < 1.00 represent development versions. From 1.00 up, they are pr
 
 L<https://github.com/ronsavage/Regexp-Parsertron>
 
-=head1 References
-
-L<https://code.activestate.com/lists/perl5-porters/209610/>
-
-L<https://stackoverflow.com/questions/46200305/a-strict-regular-expression-for-matching-chemical-formulae>
-
 =head1 Support
 
 Email the author, or log a bug on RT:
